;*************************************************************************** 
;*
;* Title:		Digital RC Transmiter to PC Joystick converter
;* Version:		1.1
;* Last updated:	14.01.2001
;* Target:		AT90S2313 
;*
;* Support E-mail:	jiv@stal.ru
;* WWW:                 http://www.rcdesign.ru
;*
;* DESCRIPTION: 
;* Digital converter, RC transmitter -> PC Joystick 
;*
;*************************************************************************** 

.include "2313def.inc"

				; Max joystick pulse length = 2000 mks
.equ	StartPulseLen = 3000	; Min start pulse length, mks
.equ	PulseMiddle   = 1500	; Channel pulse middle, mks
;.equ	MinPulseLen   = 700	; Min channel pulse length, mks
;.equ	ChDiv         = 3	; Channel data div
				; Other useful settings :
.equ	MinPulseLen   = 850	; Min channel pulse length, mks
.equ	ChDiv         = 2	; Channel data div

.equ	X1	= 1		; Joystick output pins masks
.equ	Y1	= 2
.equ	X2	= 4
.equ	Y2	= 8

.equ	Button1 = 0b00000100	; Button 1 pin mask
.equ	Button2 = 0b00001000	; Button 2 pin mask
.equ	Button3 = 0b00010000	; Button 3 pin mask
.equ	Button4 = 0b01000000	; Button 4 pin mask

.equ    Channels   = $60	; Channel data store here

;***** Global Register Variables

.def	tmr1_tempL = r3
.def	tmr1_tempH = r4
.def	icr1_tempL = r5
.def	icr1_tempH = r6

.def	Ch1tmpL    = r16	; Channel 1 - low byte
.def	Ch1tmpH    = r17	; Channel 1 - high byte
.def	Ch2tmpL	   = r18	; Channel 2 - low byte
.def	Ch2tmpH	   = r19	; Channel 2 - high byte
.def	Ch3tmpL	   = r20	; Channel 3 - low byte
.def	Ch3tmpH	   = r21	; Channel 3 - high byte
.def	Ch4tmpL	   = r22	; Channel 4 - low byte
.def	Ch4tmpH	   = r23	; Channel 4 - high byte

.def	temp	   = r24	; temporary storage register

.def	ch_state   = r30	; Channels output state temporarily store
.def	Div        = r31	; 

;*************************

.listmac
.macro	TestChannel		; 4 cycles
	sub	@0, Div		
	sbci	@1, 0		
	brge	PC+2		
	cbr	ch_state, @2
.endm

.macro	CheckButtons
	ldd	temp, Y+(2*(@0-1))
	subi	temp, low(PulseMiddle)
	ldd	temp, Y+(2*(@0-1))+1
	sbci	temp, high(PulseMiddle)
	brlo	PC+2
	sbr	ch_state, @1
.endm

.macro DirectChannel
	ldd	@0, Y+(2*(@2-1))
	ldd	@1, Y+(2*(@2-1))+1
	subi	@0, LOW(MinPulseLen)
	sbci	@1, HIGH(MinPulseLen)
.endm

.macro ReverseChannel
	ldd	@0, Y+(2*(@2-1))
	ldd	@1, Y+(2*(@2-1))+1
	com	@0
	com	@1
	subi	@0, low(-(PulseMiddle*2-MinPulseLen)-1)
	sbci	@1, high(-(PulseMiddle*2-MinPulseLen)-1)
.endm

;***** Interrupt Vectors
	rjmp	RESET			;Reset Handle
.org 	ICP1addr
	rjmp	Timer_capture_1

.org	OC1addr
	ldi	XL, Channels
	reti			; 9 cycles

.org	ACIaddr
;***** AC Interrupt Vector
	ldi	ch_state,0b00001111
	out	DDRD, ch_state		; Port D Data Direction Register  DDRD
					; Calculating channel data 1-4
	DirectChannel	Ch1tmpL,Ch1tmpH,1 ; Cannel 1
	ReverseChannel	Ch2tmpL,Ch2tmpH,2 ; Cannel 2
	ReverseChannel	Ch3tmpL,Ch3tmpH,3 ; Cannel 3
	DirectChannel	Ch4tmpL,Ch4tmpH,4 ; Cannel 4
					; Channels 5-8 output
	ldi	ch_state, 0b00000000

	CheckButtons	5, Button1	; Channel 5
	CheckButtons	6, Button2	; Channel 6
	CheckButtons	7, Button3	; Channel 7
	CheckButtons	8, Button4	; Channel 8

	ldi	Div, ChDiv
	out	DDRB,ch_state		; New buttons state output
	ldi	ch_state,0b00001111
	ldi	temp,0b00000010 	; Comparator Interrupt on Falling Output Edge
	out	ACSR, temp		; AC Interrupt disable
OutLoop:
	sei
	cli
					; Joystick pulse generation
					; 4 cycles per channel, 23 cycles total
					; if MinPulseLen   = 700:
					; ~2156 mks max time, ChDiv = 2
					; ~1438 mks max time, ChDiv = 3
					; ~1078 mks max time, ChDiv = 4
					; if MinPulseLen   = 850 :
					; ~1869 mks max time, ChDiv = 2
					
	TestChannel Ch1tmpL, Ch1tmpH, X1	; Channel 1 output 
	TestChannel Ch2tmpL, Ch2tmpH, Y1	; Channel 2 output
	TestChannel Ch3tmpL, Ch3tmpH, Y2	; Channel 3 output
	TestChannel Ch4tmpL, Ch4tmpH, X2	; Channel 4 output
					; New channell state setup
	out	DDRD, ch_state
	ldi	Div, ChDiv
	cpi	ch_state, 0b00000000	; All channels done? 
	brne	OutLoop		
	sei
WaitPulseEnd:
	sbis	ACSR, ACI
	rjmp	WaitPulseEnd

	ldi	temp,0b00011011 	; Comparator Interrupt on Rising Output Edge
	out	ACSR, temp		; AC Interrupt enable
	reti
	
;***** Timer1 Interrupt Vector
Timer_capture_1:
	in	icr1_tempL, ICR1L	; Timer 1 reloading & correction
	in	icr1_tempH, ICR1H
	in	tmr1_tempL, TCNT1L
	in	tmr1_tempH, TCNT1H
	sub	tmr1_tempL, icr1_tempL
	sbc	tmr1_tempH, icr1_tempH
	out	TCNT1H,tmr1_tempH
	out	TCNT1L,tmr1_tempL
	st	X+, icr1_tempL
	st	X+, icr1_tempH
	ldi	Div, (ChDiv * 2)
	reti				; 13+8 = 21 cycles

;***************************************************************************
;*
;* PROGRAM EXECUTION STARTS HERE
;*
;***************************************************************************

RESET:
	ldi	temp,RAMEND
	out	SPL,temp
	
	clr	XH		; Channels data pointer in X
	ldi	XL, Channels
	clr	YH		; Channels data pointer in Y
	ldi	YL, Channels+2

	ldi	temp, HIGH(StartPulseLen) ; Timer/Counter1 Output Compare Register A  OCR1AH and OCR1AL
	out	OCR1AH,temp
	ldi	temp, LOW(StartPulseLen)
	out	OCR1AL,temp

	ldi	temp,0
	out	TCCR1A,temp	; Timer/Counter1 Control Register A  TCCR1A
				; Bits 7,6  COM1A1, COM1A0: Compare Output Mode1, Bits 1 and 0
				; Bits 5..2  Res: Reserved Bits
				; Bits 1,0  PWM11, PWM10: Pulse Width Modulator Select Bits
				
	ldi	temp,0b00000010
	out	TCCR1B,temp	; Timer/Counter1 Control Register B  TCCR1B
		              	; Bits 2,1,0  CS12, CS11, CS10: Clock Select1, Bits 2, 1 and 0
				;CS12 CS11 CS10 Description
				;0    0    0    Stop, the Timer/Counter1 is stopped.
				;0    0    1    CK
				;0    1    0    CK/8
				;0    1    1    CK/64
				;1    0    0    CK/256
				;1    0    1    CK/1024
				;1    1    0    External Pin T1, falling edge
				;1    1    1    External Pin T1, rising edge
		              	; bit 4 - 
		              	; bit 5 - 
		              	; Bit 6  ICES1: Input Capture1 Edge Select
		              	; Bit 7  ICNC1: Input Capture1 Noise Canceler (4 CKs)
	
	ldi	temp,0b00000001
	out	TCCR0,temp	; Timer/Counter0 Control Register  TCCR0
				; Bits 7..3  Res: Reserved Bits
				; Bits 2,1,0  CS02, CS01, CS00: Clock Select0, Bit 2,1 and 0
				; CS02 CS01 CS00 Description
				; 0    0    0    Stop, the Timer/Counter0 is stopped.
				; 0    0    1    CK
				; 0    1    0    CK/8
				; 0    1    1    CK/64
				; 1    0    0    CK/256
				; 1    0    1    CK/1024
				; 1    1    0    External Pin T0, falling edge
				; 1    1    1    External Pin T0, rising edge

	ldi	temp,0b01001000		
	out	TIMSK,temp	; Timer/Counter Interrupt Mask Register  TIMSK
				; Bit 7  TOIE1: Timer/Counter1 Overflow Interrupt Enable
				; Bit 6  OCIE1A: Timer/Counter1 Output Compare Match Interrupt Enable
				; Bit 5,4  Res: Reserved Bits
				; Bit 3  TICIE1: Timer/Counter1 Input Capture Interrupt Enable
				; Bit 2  Res: Reserved Bit
				; Bit 1  TOIE0: Timer/Counter0 Overflow Interrupt Enable
				; Bit 0  Res: Reserved Bit

	; Analog Comparator setings
	ldi	temp,0b00001011
	out	ACSR,temp	; Analog Comparator Control and Status Register
				; Bit 7  ACD: Analog Comparator Disable
				; Bit 6  Res: Reserved Bit
				; Bit 5  ACO: Analog Comparator Output
				; Bit 4  ACI: Analog Comparator Interrupt Flag
				; Bit 3  ACIE: Analog Comparator Interrupt Enable
				; Bit 2  ACIC: Analog Comparator Input Capture Enable
				; Bits 1,0  ACIS1, ACIS0: Analog Comparator Interrupt Mode Select
				; 	0 0 Comparator Interrupt on Output Toggle
				; 	0 1 Reserved
				; 	1 0 Comparator Interrupt on Falling Output Edge
				; 	1 1 Comparator Interrupt on Rising Output Edge
	
				; All buttons released
	ldi	temp,0b00000000	
	out	DDRB,temp	; Port B Data Direction Register  DDRD
	ldi	temp,0b00000000	
	out	PORTB,temp	; Port B Data Register
				
	ldi	temp,0b00000000		
	out	DDRD,temp	; Port D Data Direction Register  DDRD
	ldi	temp,0b01000000 		
	out	PORTD,temp	; Port D, disable pull-up resistors on channels output
				; enable pull-up resistors on TX input

				; data memory
	ldi	temp, low(PulseMiddle)
	std	Y+0,  temp
	std	Y+2,  temp
	std	Y+4,  temp
	std	Y+6,  temp
	ldi	temp, high(PulseMiddle)
	std	Y+1,  temp
	std	Y+3,  temp
	std	Y+5,  temp
	std	Y+7,  temp
	ldi	temp, low(MinPulseLen)
	std	Y+8,  temp
	std	Y+10, temp
	std	Y+12, temp
	std	Y+14, temp
	ldi	temp, high(MinPulseLen)
	std	Y+9,  temp
	std	Y+11, temp
	std	Y+13, temp
	std	Y+15, temp

	sei			; Enable Interrupt

MainLoop:
	rjmp	MainLoop
